@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -373,7 +373,16 @@ public final class POP3Exception extends MIMEMailException {
         /**
          * Missing required capability %1$s on server %2$s with login %3$s (user=%4$s, context=%5$s).
          */
-        MISSING_REQUIRED_CAPABILITY("Missing required capability %1$s on server %2$s with login %3$s (user=%4$s, context=%5$s).", Category.SETUP_ERROR, 2068);
+        MISSING_REQUIRED_CAPABILITY("Missing required capability %1$s on server %2$s with login %3$s (user=%4$s, context=%5$s).", Category.SETUP_ERROR, 2068),
+        /**
+         * POP3 storage path "%1$s" cannot be created for user %2$s in context %3$s.
+         */
+        ILLEGAL_PATH("POP3 storage path \"%1$s\" cannot be created for user %2$s in context %3$s.", Category.CODE_ERROR, 2069),
+        /**
+         * Due to missing required capability %1$s POP3 messages are fetched and removed (expunge-on-quit) from server %2$s with login %3$s
+         * (user=%4$s, context=%5$s).
+         */
+        EXPUNGE_MODE_ONLY("Due to missing required capability %1$s POP3 messages are fetched and removed (expunge-on-quit) from server %2$s with login %3$s (user=%4$s, context=%5$s).", Category.SETUP_ERROR, 2070);
 
         private final String message;
 
