@@ -49,6 +49,7 @@
 
 package com.openexchange.pop3.storage.mailaccount;
 
+import java.security.SecureRandom;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -140,9 +141,9 @@ public class MailAccountPOP3Storage implements POP3Storage {
                         POP3Exception.Code.MISSING_PATH,
                         Integer.valueOf(session.getUserId()),
                         Integer.valueOf(session.getContextId()));
-                LOG.warn("Path is null. Error:" + e.getMessage(), e);
+                LOG.warn("Path is null. Error: " + e.getMessage(), e);
                 // Try to compose path
-                tmp = composePath(pop3Access.getAccountId(), session.getUserId(), session.getContextId());
+                tmp = composeUniquePath(pop3Access.getAccountId(), session.getUserId(), session.getContextId());
                 // Add to properties
                 properties.addProperty(POP3StoragePropertyNames.PROPERTY_PATH, tmp);
             }
@@ -157,31 +158,71 @@ public class MailAccountPOP3Storage implements POP3Storage {
         separator = 0;
     }
 
-    private String composePath(final int pop3AccountId, final int user, final int cid) throws MailException {
+    private String composeUniquePath(final int pop3AccountId, final int user, final int cid) throws MailException {
         defaultMailAccess.connect(false);
         try {
-            final String fn = defaultMailAccess.getFolderStorage().getTrashFolder();
+            final String trashFullname = defaultMailAccess.getFolderStorage().getTrashFolder();
             final char sep = defaultMailAccess.getFolderStorage().getFolder("INBOX").getSeparator();
-            final int pos = fn.lastIndexOf(sep);
-            final String accoutnName;
+            /*
+             * Check location of trash folder: beside or below INBOX folder?
+             */
+            final int pos = trashFullname.lastIndexOf(sep);
+            final String accountName;
             try {
                 final MailAccountStorageService storageService =
                     POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
-                accoutnName = storageService.getMailAccount(pop3AccountId, user, cid).getName();
+                accountName = stripSpecials(storageService.getMailAccount(pop3AccountId, user, cid).getName());
             } catch (final ServiceException e) {
                 throw new MailException(e);
             } catch (final MailAccountException e) {
                 throw new MailException(e);
             }
+            String fullname;
             if (pos == -1) {
-                return accoutnName;
+                /*
+                 * Beside INBOX folder
+                 */
+                fullname = accountName;
+            } else {
+                /*
+                 * Below INBOX folder but beside trash folder
+                 */
+                fullname = new StringBuilder(16).append(trashFullname.substring(0, pos)).append(sep).append(accountName).toString();
             }
-            return new StringBuilder(16).append(fn.substring(0, pos)).append(sep).append(accoutnName).toString();
+            /*
+             * Check existence
+             */
+            if (defaultMailAccess.getFolderStorage().exists(fullname)) {
+                final String pre = fullname;
+                final SecureRandom secureRandom = new SecureRandom();
+                do {
+                    fullname = pre + stripSpecials(String.valueOf(secureRandom.nextLong()));
+                } while (defaultMailAccess.getFolderStorage().exists(fullname));
+            }
+            /*
+             * Return unique path
+             */
+            return fullname;
         } finally {
             defaultMailAccess.close(true);
         }
     }
 
+    private static String stripSpecials(final String src) {
+        if (null == src || src.length() == 0) {
+            return String.valueOf(System.currentTimeMillis());
+        }
+        final char[] chars = src.toCharArray();
+        final StringBuilder sb = new StringBuilder(chars.length);
+        for (int i = 0; i < chars.length; i++) {
+            char c = chars[i];
+            if (Character.isLetterOrDigit(c)) {
+                sb.append(c);
+            }
+        }
+        return sb.toString();
+    }
+
     public Collection<MailException> getWarnings() {
         return Collections.unmodifiableCollection(warnings);
     }
@@ -208,7 +249,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
         return path;
     }
 
-    public void close() throws MailException {
+    public void close() {
         defaultMailAccess.close(true);
     }
 
@@ -231,23 +272,29 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
                 toCreate.addPermission(mp);
                 toCreate.setExists(false);
-                
+
                 /*
                  * Determine where to create
                  */
 
-                final char separator = defaultMailAccess.getFolderStorage().getFolder("INBOX").getSeparator();
-
+                final MailFolder inboxFolder = defaultMailAccess.getFolderStorage().getFolder("INBOX");
+                final char separator = inboxFolder.getSeparator();
                 final String[] parentAndName = parseFullname(path, separator);
                 /*
                  * Check CREATE permission
                  */
-                final MailPermission ownPermission = fs.getFolder(parentAndName[0]).getOwnPermission();
-                if (ownPermission.canCreateSubfolders()) {
+                final MailPermission ownPermission;
+                final String parentFullname = parentAndName[0];
+                if ("INBOX".equals(parentFullname)) {
+                    ownPermission = inboxFolder.getOwnPermission();
+                } else {
+                    ownPermission = fs.getFolder(parentFullname).getOwnPermission();
+                }
+                if (null ==ownPermission || ownPermission.canCreateSubfolders()) {
                     /*
                      * Set parent to current path's parent
                      */
-                    toCreate.setParentFullname(parentAndName[0]);
+                    toCreate.setParentFullname(parentFullname);
                 } else {
                     /*
                      * Path is invalid! Change path
@@ -271,6 +318,10 @@ public class MailAccountPOP3Storage implements POP3Storage {
                     // Update in properties
                     properties.addProperty(POP3StoragePropertyNames.PROPERTY_PATH, path);
                     if (fs.exists(path)) {
+                        /*
+                         * Check default folders
+                         */
+                        getFolderStorage().checkDefaultFolders();
                         return;
                     }
                 }
@@ -283,9 +334,18 @@ public class MailAccountPOP3Storage implements POP3Storage {
                 try {
                     fs.createFolder(toCreate);
                 } catch (final MailException e) {
-                    throw new POP3Exception(POP3Exception.Code.ILLEGAL_PATH, e, path, Integer.valueOf(session.getUserId()),
+                    throw new POP3Exception(
+                        POP3Exception.Code.ILLEGAL_PATH,
+                        e,
+                        path,
+                        Integer.valueOf(session.getUserId()),
                         Integer.valueOf(session.getContextId()));
                 }
+
+                /*
+                 * Check default folders
+                 */
+                getFolderStorage().checkDefaultFolders();
             }
         } catch (final MailException e) {
             /*
@@ -368,19 +428,25 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     private static final Flags FLAGS_DELETED = new Flags(Flags.Flag.DELETED);
 
-    public void syncMessages(final boolean expunge, final POP3StorageConnectCounter connectCounter) throws MailException {
-        final POP3StoreResult result =
-            POP3StoreConnector.getPOP3Store(pop3Access.getPOP3Config(), pop3Access.getMailProperties(), false, pop3Access.getSession(), !expunge);
-        final POP3Store pop3Store = result.getPop3Store();
-        final boolean containsWarnings = result.containsWarnings();
-        if (containsWarnings) {
-            this.warnings.addAll(result.getWarnings());
-        }
-        /*
-         * Increase counter
-         */
-        connectCounter.incrementCounter();
+    public void syncMessages(final boolean expunge, final POP3StorageConnectCounter connectCounter) {
+        POP3Store pop3Store = null;
         try {
+            final POP3StoreResult result =
+                POP3StoreConnector.getPOP3Store(
+                    pop3Access.getPOP3Config(),
+                    pop3Access.getMailProperties(),
+                    false,
+                    pop3Access.getSession(),
+                    !expunge);
+            pop3Store = result.getPop3Store();
+            final boolean containsWarnings = result.containsWarnings();
+            if (containsWarnings) {
+                this.warnings.addAll(result.getWarnings());
+            }
+            /*
+             * Increase counter
+             */
+            connectCounter.incrementCounter();
             final POP3Folder inbox = (POP3Folder) pop3Store.getFolder("INBOX");
             inbox.open(POP3Folder.READ_WRITE);
             boolean doExpunge = false;
@@ -391,6 +457,12 @@ public class MailAccountPOP3Storage implements POP3Storage {
                     final Message[] all = inbox.getMessages();
                     if (containsWarnings) {
                         addAllMessagesToStorage(inbox, all);
+                        /*
+                         * Mark messages as \Deleted
+                         */
+                        for (int i = 0; i < all.length; i++) {
+                            all[i].setFlags(FLAGS_DELETED, true);
+                        }
                         doExpunge = true;
                     } else {
                         // Initiate UIDL fetch
@@ -421,7 +493,9 @@ public class MailAccountPOP3Storage implements POP3Storage {
                         addMessagesToStorage(newUIDLs, inbox, all);
 
                         if (expunge) {
-                            // Mark messages as \Deleted
+                            /*
+                             * Mark messages as \Deleted
+                             */
                             for (int i = 0; i < all.length; i++) {
                                 all[i].setFlags(FLAGS_DELETED, true);
                             }
@@ -449,42 +523,49 @@ public class MailAccountPOP3Storage implements POP3Storage {
                 }
             }
         } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e, pop3Access.getPOP3Config(), pop3Access.getSession());
+            MailException e1 = MIMEMailException.handleMessagingException(e, pop3Access.getPOP3Config(), pop3Access.getSession());
+            LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
+            warnings.add(e1);
+        } catch (MailException e) {
+            LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
+            warnings.add(e);
         } finally {
             try {
-                pop3Store.close();
+                if (null != pop3Store) {
+                    pop3Store.close();
+                }
             } catch (final MessagingException e) {
                 LOG.error(e.getMessage(), e);
             } finally {
-                /*
-                 * Decrease counters
-                 */
                 connectCounter.decrementCounter();
             }
         }
     }
 
     private void addMessagesToStorage(final Set<String> newUIDLs, final POP3Folder inbox, final Message[] all) throws MessagingException, MailException {
-        /*
-         * Filter new ones
-         */
-        final List<Message> toFetch = new ArrayList<Message>(newUIDLs.size());
-        for (int i = 0; i < all.length; i++) {
-            final Message message = all[i];
-            final String uidl = inbox.getUID(message);
-            if (newUIDLs.contains(uidl)) {
-                toFetch.add(message);
+        final Message[] msgs;
+        {
+            /*
+             * Filter new ones
+             */
+            final List<Message> toFetch = new ArrayList<Message>(newUIDLs.size());
+            for (int i = 0; i < all.length; i++) {
+                final Message message = all[i];
+                final String uidl = inbox.getUID(message);
+                if (newUIDLs.contains(uidl)) {
+                    toFetch.add(message);
+                }
             }
+            /*
+             * Fetch ENVELOPE for new messages
+             */
+            msgs = toFetch.toArray(new Message[toFetch.size()]);
+            final FetchProfile fetchProfile = new FetchProfile();
+            fetchProfile.add(FetchProfile.Item.ENVELOPE);
+            final long start = System.currentTimeMillis();
+            inbox.fetch(msgs, fetchProfile);
+            MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
         }
-        /*
-         * Fetch ENVELOPE for new messages
-         */
-        final Message[] msgs = toFetch.toArray(new Message[toFetch.size()]);
-        final FetchProfile fetchProfile = new FetchProfile();
-        fetchProfile.add(FetchProfile.Item.ENVELOPE);
-        final long start = System.currentTimeMillis();
-        inbox.fetch(msgs, fetchProfile);
-        MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
         /*
          * Append them to storage
          */
